"""Sentence Transformers embeddings provider implementation."""

from typing import TYPE_CHECKING

from .base import EmbeddingProvider

if TYPE_CHECKING:
    from sentence_transformers import SentenceTransformer

# Flag to check if sentence-transformers is available
SENTENCE_TRANSFORMERS_AVAILABLE = False
IMPORT_ERROR_MSG = """
sentence-transformers is not installed. To use local embeddings, install it with:

    pip install sentence-transformers

Or if using uvx:

    uvx --with sentence-transformers qdrant-mcp
"""

try:
    from sentence_transformers import SentenceTransformer
    SENTENCE_TRANSFORMERS_AVAILABLE = True
except ImportError:
    pass


class SentenceTransformersEmbeddingProvider(EmbeddingProvider):
    """Sentence Transformers embeddings provider for local embeddings."""
    
    # Model dimensions mapping
    MODEL_DIMENSIONS = {
        "all-MiniLM-L6-v2": 384,
        "all-mpnet-base-v2": 768,
    }
    
    def __init__(self, model_name: str = "all-MiniLM-L6-v2", device: str | None = None):
        """Initialize Sentence Transformers embedding provider.
        
        Args:
            model_name: Name of the sentence transformer model
            device: Device to run the model on (cpu, cuda, etc.)
        """
        if not SENTENCE_TRANSFORMERS_AVAILABLE:
            raise ImportError(IMPORT_ERROR_MSG)
        
        if model_name not in self.MODEL_DIMENSIONS:
            # Try to load it anyway - sentence transformers has many models
            # Get dimensions after loading
            self.model = SentenceTransformer(model_name, device=device)
            # Encode a dummy text to get dimensions
            dummy_embedding = self.model.encode("test", convert_to_numpy=True)
            dimensions = len(dummy_embedding)
        else:
            dimensions = self.MODEL_DIMENSIONS[model_name]
            self.model = SentenceTransformer(model_name, device=device)
        
        super().__init__(model_name, dimensions)
    
    async def embed_text(self, text: str) -> list[float]:
        """Embed a single text using Sentence Transformers.
        
        Args:
            text: Text to embed
            
        Returns:
            Embedding vector
        """
        # Convert to numpy array and then to list
        embedding = self.model.encode(text, convert_to_numpy=True)
        return embedding.tolist()
    
    async def embed_batch(self, texts: list[str]) -> list[list[float]]:
        """Embed multiple texts using Sentence Transformers.
        
        Args:
            texts: List of texts to embed
            
        Returns:
            List of embedding vectors
        """
        if not texts:
            return []
        
        # Batch encode for efficiency
        embeddings = self.model.encode(texts, convert_to_numpy=True, show_progress_bar=False)
        
        # Convert numpy array to list of lists
        return embeddings.tolist()
    
    @property
    def provider_name(self) -> str:
        """Get the provider name."""
        return "sentence-transformers"


"""Base class for embedding providers."""

from abc import ABC, abstractmethod
from typing import Any


class EmbeddingProvider(ABC):
    """Abstract base class for embedding providers."""
    
    def __init__(self, model_name: str, dimensions: int):
        """Initialize the embedding provider.
        
        Args:
            model_name: Name of the embedding model
            dimensions: Dimension of the embeddings
        """
        self.model_name = model_name
        self.dimensions = dimensions
    
    @abstractmethod
    async def embed_text(self, text: str) -> list[float]:
        """Embed a single text string.
        
        Args:
            text: Text to embed
            
        Returns:
            Embedding vector
        """
        pass
    
    @abstractmethod
    async def embed_batch(self, texts: list[str]) -> list[list[float]]:
        """Embed multiple texts in batch.
        
        Args:
            texts: List of texts to embed
            
        Returns:
            List of embedding vectors
        """
        pass
    
    @property
    @abstractmethod
    def provider_name(self) -> str:
        """Get the name of the embedding provider."""
        pass
    
    def get_model_info(self) -> dict[str, Any]:
        """Get information about the embedding model.
        
        Returns:
            Dictionary with model information
        """
        return {
            "provider": self.provider_name,
            "model": self.model_name,
            "dimensions": self.dimensions
        }


"""Qdrant client wrapper with embedding support."""

import uuid
from datetime import datetime
from typing import Any

from qdrant_client import AsyncQdrantClient
from qdrant_client.models import (
    Distance,
    FieldCondition,
    Filter,
    MatchValue,
    PointStruct,
    VectorParams,
)

from .embeddings import create_embedding_provider
from .settings import Settings


class QdrantMemoryClient:
    """Wrapper around Qdrant client with embedding support."""
    
    def __init__(self, settings: Settings):
        """Initialize Qdrant client with settings.
        
        Args:
            settings: Application settings
        """
        self.settings = settings
        # Parse URL to extract host and port
        from urllib.parse import urlparse
        parsed_url = urlparse(settings.qdrant_url)
        host = parsed_url.hostname
        port = parsed_url.port
        https = parsed_url.scheme == "https"
        
        # Use host/port parameters instead of URL for better compatibility
        if host:
            self.client = AsyncQdrantClient(
                host=host,
                port=port or (443 if https else 6333),
                https=https,
                api_key=settings.qdrant_api_key,
                timeout=30.0,  # Increased timeout for remote server
                prefer_grpc=False,  # Use REST API for better HTTPS support
            )
        else:
            # Fallback to URL parameter
            self.client = AsyncQdrantClient(
                url=settings.qdrant_url,
                api_key=settings.qdrant_api_key,
                timeout=30.0,  # Increased timeout for remote server
                prefer_grpc=False,  # Use REST API for better HTTPS support
            )
        
        # Create embedding provider
        self.embedding_provider = create_embedding_provider(
            provider=settings.embedding_provider,
            model_name=settings.embedding_model,
            api_key=settings.openai_api_key,
            device=settings.device,
        )
        
        # Initialize collection flag
        self._collection_initialized = False
    
    async def _ensure_collection(self) -> None:
        """Ensure the collection exists (lazy initialization)."""
        if self._collection_initialized:
            return
            
        try:
            collections = await self.client.get_collections()
            collection_names = [c.name for c in collections.collections]
            
            if self.settings.collection_name not in collection_names:
                # Create collection with appropriate vector size
                await self.client.create_collection(
                    collection_name=self.settings.collection_name,
                    vectors_config=VectorParams(
                        size=self.embedding_provider.dimensions,
                        distance=Distance.COSINE,
                ),
            )
            self._collection_initialized = True
        except Exception as e:
            raise RuntimeError(f"Failed to initialize Qdrant collection: {e}")
    
    async def store(
        self,
        content: str,
        metadata: dict[str, Any] | None = None,
        id: str | None = None
    ) -> str:
        """Store content with embeddings in Qdrant.
        
        Args:
            content: Text content to store
            metadata: Optional metadata to attach
            id: Optional ID for the point (generated if not provided)
            
        Returns:
            ID of the stored point
        """
        # Ensure collection exists
        await self._ensure_collection()
        
        # Generate ID if not provided
        point_id = id or str(uuid.uuid4())
        
        # Generate embedding
        embedding = await self.embedding_provider.embed_text(content)
        
        # Prepare payload
        payload = {
            "content": content,
            "timestamp": datetime.utcnow().isoformat(),
            "embedding_model": self.embedding_provider.model_name,
            "embedding_provider": self.embedding_provider.provider_name,
        }
        
        if metadata:
            payload["metadata"] = metadata
        
        # Create point
        point = PointStruct(
            id=point_id,
            vector=embedding,
            payload=payload,
        )
        
        # Upsert to Qdrant
        await self.client.upsert(
            collection_name=self.settings.collection_name,
            points=[point],
        )
        
        return point_id
    
    async def find(
        self,
        query: str,
        limit: int | None = None,
        filter: dict[str, Any] | None = None,
        score_threshold: float | None = None
    ) -> list[dict[str, Any]]:
        """Find similar content using semantic search.
        
        Args:
            query: Search query text
            limit: Number of results to return
            filter: Optional filter conditions
            score_threshold: Minimum score threshold
            
        Returns:
            List of search results with content and metadata
        """
        # Ensure collection exists
        await self._ensure_collection()
        
        # Use defaults from settings if not provided
        limit = limit or self.settings.default_limit
        score_threshold = score_threshold or self.settings.score_threshold
        
        # Generate query embedding
        query_embedding = await self.embedding_provider.embed_text(query)
        
        # Build filter if provided
        search_filter = None
        if filter:
            conditions = []
            for key, value in filter.items():
                conditions.append(
                    FieldCondition(
                        key=f"metadata.{key}",
                        match=MatchValue(value=value),
                    )
                )
            if conditions:
                search_filter = Filter(must=conditions)
        
        # Search
        results = await self.client.search(
            collection_name=self.settings.collection_name,
            query_vector=query_embedding,
            limit=limit,
            query_filter=search_filter,
            score_threshold=score_threshold,
            with_payload=True,
            with_vectors=False,
        )
        
        # Format results
        formatted_results = []
        for result in results:
            formatted_result = {
                "id": result.id,
                "score": result.score,
                "content": result.payload.get("content", ""),
                "timestamp": result.payload.get("timestamp", ""),
                "metadata": result.payload.get("metadata", {}),
                "embedding_model": result.payload.get("embedding_model", ""),
                "embedding_provider": result.payload.get("embedding_provider", ""),
            }
            formatted_results.append(formatted_result)
        
        return formatted_results
    
    async def delete(self, ids: list[str]) -> dict[str, Any]:
        """Delete points by IDs.
        
        Args:
            ids: List of point IDs to delete
            
        Returns:
            Operation result
        """
        # Ensure collection exists
        await self._ensure_collection()
        
        await self.client.delete(
            collection_name=self.settings.collection_name,
            points_selector=ids,
        )
        
        return {
            "deleted": len(ids),
            "ids": ids,
        }
    
    async def list_collections(self) -> list[str]:
        """List all collections in Qdrant.
        
        Returns:
            List of collection names
        """
        collections = await self.client.get_collections()
        return [c.name for c in collections.collections]
    
    async def get_collection_info(self) -> dict[str, Any]:
        """Get information about the current collection.
        
        Returns:
            Collection information
        """
        # Ensure collection exists
        await self._ensure_collection()
        
        info = await self.client.get_collection(self.settings.collection_name)
        return {
            "name": self.settings.collection_name,
            "vectors_count": info.vectors_count,
            "points_count": info.points_count,
            "vector_size": info.config.params.vectors.size,
            "distance": info.config.params.vectors.distance,
        }
    
    async def close(self) -> None:
        """Close connections and cleanup."""
        if hasattr(self.embedding_provider, "close"):
            await self.embedding_provider.close()
        await self.client.close()


"""Settings management for Qdrant MCP server."""

from typing import Literal

from pydantic import Field, field_validator
from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    """Configuration settings for Qdrant MCP server."""
    
    model_config = SettingsConfigDict(
        env_file=".env",
        env_file_encoding="utf-8",
        case_sensitive=False,
        extra="ignore",
    )
    
    # Qdrant settings
    qdrant_url: str = Field(
        default="http://localhost:6333",
        description="URL of the Qdrant server"
    )
    qdrant_api_key: str | None = Field(
        default=None,
        description="API key for Qdrant (optional)"
    )
    collection_name: str = Field(
        default="mcp_memory",
        description="Name of the Qdrant collection to use"
    )
    
    # Embedding settings
    embedding_provider: Literal["openai", "sentence-transformers"] = Field(
        default="openai",
        description="Embedding provider to use"
    )
    embedding_model: str = Field(
        default="text-embedding-3-small",
        description="Embedding model name"
    )
    
    # OpenAI settings (optional, used when provider is "openai")
    openai_api_key: str | None = Field(
        default=None,
        description="OpenAI API key (required for OpenAI embeddings)"
    )
    
    # Sentence Transformers settings (optional)
    device: str | None = Field(
        default=None,
        description="Device to run sentence transformers on (cpu, cuda, etc.)"
    )
    
    # Search settings
    default_limit: int = Field(
        default=10,
        ge=1,
        le=100,
        description="Default number of results to return"
    )
    score_threshold: float = Field(
        default=0.0,
        ge=0.0,
        le=1.0,
        description="Minimum score threshold for search results"
    )
    
    # Server settings
    server_name: str = Field(
        default="qdrant-mcp",
        description="Name of the MCP server"
    )
    server_version: str = Field(
        default="0.1.0",
        description="Version of the MCP server"
    )
    
    @field_validator("embedding_model")
    @classmethod
    def validate_embedding_model(cls, v: str, info) -> str:
        """Validate embedding model based on provider."""
        provider = info.data.get("embedding_provider", "openai")
        
        # Define valid models per provider
        valid_models = {
            "openai": [
                "text-embedding-3-small",
                "text-embedding-3-large",
                "text-embedding-ada-002"
            ],
            "sentence-transformers": [
                "all-MiniLM-L6-v2",
                "all-mpnet-base-v2",
                # Allow any model for sentence-transformers as it supports many
            ]
        }
        
        if provider == "openai" and v not in valid_models["openai"]:
            raise ValueError(
                f"Invalid OpenAI model: {v}. "
                f"Valid models: {', '.join(valid_models['openai'])}"
            )
        
        return v
    
    @field_validator("openai_api_key")
    @classmethod
    def validate_openai_api_key(cls, v: str | None, info) -> str | None:
        """Validate OpenAI API key is provided when using OpenAI provider."""
        provider = info.data.get("embedding_provider", "openai")
        
        if provider == "openai" and not v:
            raise ValueError(
                "OpenAI API key is required when using OpenAI embedding provider. "
                "Set OPENAI_API_KEY environment variable."
            )
        
        return v


def get_settings() -> Settings:
    """Get settings instance."""
    return Settings()


"""Tests for settings management."""

import pytest
from unittest.mock import patch

from qdrant_mcp.settings import Settings


class TestSettings:
    """Test settings configuration."""
    
    def test_default_settings(self):
        """Test default settings values."""
        with patch.dict("os.environ", {"OPENAI_API_KEY": "test-key"}):
            settings = Settings()
            assert settings.qdrant_url == "http://localhost:6333"
            assert settings.collection_name == "mcp_memory"
            assert settings.embedding_provider == "openai"
            assert settings.embedding_model == "text-embedding-3-small"
            assert settings.default_limit == 10
    
    def test_environment_variables(self):
        """Test loading settings from environment."""
        env_vars = {
            "QDRANT_URL": "http://custom:6333",
            "COLLECTION_NAME": "custom_collection",
            "EMBEDDING_PROVIDER": "sentence-transformers",
            "EMBEDDING_MODEL": "all-MiniLM-L6-v2",
            "DEFAULT_LIMIT": "20",
        }
        
        with patch.dict("os.environ", env_vars):
            settings = Settings()
            assert settings.qdrant_url == "http://custom:6333"
            assert settings.collection_name == "custom_collection"
            assert settings.embedding_provider == "sentence-transformers"
            assert settings.embedding_model == "all-MiniLM-L6-v2"
            assert settings.default_limit == 20
    
    def test_openai_api_key_validation(self):
        """Test OpenAI API key validation."""
        # Should raise error when using OpenAI without API key
        with patch.dict("os.environ", {}, clear=True):
            with pytest.raises(ValueError, match="OpenAI API key is required"):
                Settings(embedding_provider="openai")
        
        # Should not raise error for sentence-transformers
        with patch.dict("os.environ", {}, clear=True):
            settings = Settings(embedding_provider="sentence-transformers")
            assert settings.openai_api_key is None
    
    def test_model_validation(self):
        """Test embedding model validation."""
        with patch.dict("os.environ", {"OPENAI_API_KEY": "test-key"}):
            # Valid OpenAI model
            settings = Settings(
                embedding_provider="openai",
                embedding_model="text-embedding-3-large"
            )
            assert settings.embedding_model == "text-embedding-3-large"
            
            # Invalid OpenAI model
            with pytest.raises(ValueError, match="Invalid OpenAI model"):
                Settings(
                    embedding_provider="openai",
                    embedding_model="invalid-model"
                )


"""MCP server implementation for Qdrant."""

import json
import logging
import os
import sys
from contextlib import asynccontextmanager
from typing import Any

from mcp.server import FastMCP

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from qdrant_mcp.qdrant_memory import QdrantMemoryClient
from qdrant_mcp.settings import get_settings

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Initialize Qdrant client (will be created on startup)
qdrant_client: QdrantMemoryClient | None = None


@asynccontextmanager
async def lifespan(app: FastMCP):
    """Manage the lifecycle of the Qdrant client."""
    global qdrant_client
    try:
        # Startup
        settings = get_settings()
        qdrant_client = QdrantMemoryClient(settings)
        logger.info("Qdrant MCP server initialized")
        logger.info(f"Qdrant URL: {settings.qdrant_url}")
        logger.info(f"Collection: {settings.collection_name}")
        logger.info(f"Embedding: {settings.embedding_provider} / {settings.embedding_model}")
        yield
    except Exception as e:
        logger.error(f"Failed to initialize Qdrant client: {e}")
        raise
    finally:
        # Shutdown
        if qdrant_client:
            await qdrant_client.close()
            logger.info("Qdrant client closed")


# Initialize MCP with lifespan
mcp = FastMCP("qdrant-mcp", lifespan=lifespan)


@mcp.tool()
async def qdrant_store(content: str, metadata: str | None = None, id: str | None = None) -> str:
    """Store information in Qdrant with semantic embeddings.
    
    Args:
        content: The text content to store
        metadata: Optional JSON string with metadata
        id: Optional ID for the stored item
        
    Returns:
        ID of the stored item
    """
    global qdrant_client
    if not qdrant_client:
        raise RuntimeError("Qdrant client not initialized")
    
    # Parse metadata if provided
    metadata_dict = None
    if metadata:
        try:
            metadata_dict = json.loads(metadata)
        except json.JSONDecodeError:
            raise ValueError("Metadata must be valid JSON")
    
    # Store in Qdrant
    point_id = await qdrant_client.store(
        content=content,
        metadata=metadata_dict,
        id=id
    )
    
    return f"Stored successfully with ID: {point_id}"


@mcp.tool()
async def qdrant_find(
    query: str,
    limit: int | None = None,
    filter: str | None = None,
    score_threshold: float | None = None
) -> list[dict[str, Any]]:
    """Find relevant information using semantic search.
    
    Args:
        query: Search query text
        limit: Maximum number of results to return
        filter: Optional JSON string with filter conditions
        score_threshold: Minimum similarity score (0-1)
        
    Returns:
        List of matching results with content and metadata
    """
    global qdrant_client
    if not qdrant_client:
        raise RuntimeError("Qdrant client not initialized")
    
    # Parse filter if provided
    filter_dict = None
    if filter:
        try:
            filter_dict = json.loads(filter)
        except json.JSONDecodeError:
            raise ValueError("Filter must be valid JSON")
    
    # Search in Qdrant
    results = await qdrant_client.find(
        query=query,
        limit=limit,
        filter=filter_dict,
        score_threshold=score_threshold
    )
    
    return results


@mcp.tool()
async def qdrant_delete(ids: str) -> dict[str, Any]:
    """Delete items from Qdrant by their IDs.
    
    Args:
        ids: Comma-separated list of IDs to delete
        
    Returns:
        Deletion result
    """
    global qdrant_client
    if not qdrant_client:
        raise RuntimeError("Qdrant client not initialized")
    
    # Parse IDs
    id_list = [id.strip() for id in ids.split(",") if id.strip()]
    
    if not id_list:
        raise ValueError("No IDs provided")
    
    # Delete from Qdrant
    result = await qdrant_client.delete(id_list)
    
    return result


@mcp.tool()
async def qdrant_list_collections() -> list[str]:
    """List all collections in the Qdrant database.
    
    Returns:
        List of collection names
    """
    global qdrant_client
    if not qdrant_client:
        raise RuntimeError("Qdrant client not initialized")
    
    return await qdrant_client.list_collections()


@mcp.tool()
async def qdrant_collection_info() -> dict[str, Any]:
    """Get information about the current collection.
    
    Returns:
        Collection statistics and configuration
    """
    global qdrant_client
    if not qdrant_client:
        raise RuntimeError("Qdrant client not initialized")
    
    return await qdrant_client.get_collection_info()


def main():
    """Main entry point for the MCP server."""
    # Run the server
    mcp.run()


if __name__ == "__main__":
    main()


"""Factory for creating embedding providers."""

from typing import Any

from .base import EmbeddingProvider
from .openai import OpenAIEmbeddingProvider
from .sentence_transformers import (
    IMPORT_ERROR_MSG,
    SENTENCE_TRANSFORMERS_AVAILABLE,
    SentenceTransformersEmbeddingProvider,
)


def create_embedding_provider(
    provider: str,
    model_name: str,
    **kwargs: Any
) -> EmbeddingProvider:
    """Create an embedding provider instance.
    
    Args:
        provider: Provider name ("openai" or "sentence-transformers")
        model_name: Model name for the provider
        **kwargs: Additional provider-specific arguments
        
    Returns:
        EmbeddingProvider instance
        
    Raises:
        ValueError: If provider is not supported
    """
    provider = provider.lower()
    
    if provider == "openai":
        return OpenAIEmbeddingProvider(
            model_name=model_name,
            api_key=kwargs.get("api_key")
        )
    elif provider == "sentence-transformers" or provider == "sentence_transformers":
        if not SENTENCE_TRANSFORMERS_AVAILABLE:
            raise ImportError(IMPORT_ERROR_MSG)
        return SentenceTransformersEmbeddingProvider(
            model_name=model_name,
            device=kwargs.get("device")
        )
    else:
        raise ValueError(
            f"Unknown embedding provider: {provider}. "
            f"Supported providers: openai, sentence-transformers"
        )


def get_supported_models() -> dict[str, dict[str, Any]]:
    """Get information about all supported models.
    
    Returns:
        Dictionary with model information
    """
    return {
        "openai": {
            "text-embedding-3-small": {"dimensions": 1536, "default": True},
            "text-embedding-3-large": {"dimensions": 3072},
            "text-embedding-ada-002": {"dimensions": 1536, "legacy": True},
        },
        "sentence-transformers": {
            "all-MiniLM-L6-v2": {"dimensions": 384, "default": True},
            "all-mpnet-base-v2": {"dimensions": 768},
        }
    }


"""Tests for embedding providers."""

import pytest
from unittest.mock import Mock, AsyncMock, patch

from qdrant_mcp.embeddings import create_embedding_provider
from qdrant_mcp.embeddings.base import EmbeddingProvider
from qdrant_mcp.embeddings.openai import OpenAIEmbeddingProvider
from qdrant_mcp.embeddings.sentence_transformers import SentenceTransformersEmbeddingProvider


class TestEmbeddingFactory:
    """Test embedding provider factory."""
    
    def test_create_openai_provider(self):
        """Test creating OpenAI provider."""
        with patch.dict("os.environ", {"OPENAI_API_KEY": "test-key"}):
            provider = create_embedding_provider(
                provider="openai",
                model_name="text-embedding-3-small"
            )
            assert isinstance(provider, OpenAIEmbeddingProvider)
            assert provider.model_name == "text-embedding-3-small"
            assert provider.dimensions == 1536
    
    def test_create_sentence_transformers_provider(self):
        """Test creating Sentence Transformers provider."""
        with patch("sentence_transformers.SentenceTransformer"):
            provider = create_embedding_provider(
                provider="sentence-transformers",
                model_name="all-MiniLM-L6-v2"
            )
            assert isinstance(provider, SentenceTransformersEmbeddingProvider)
            assert provider.model_name == "all-MiniLM-L6-v2"
    
    def test_invalid_provider(self):
        """Test invalid provider raises error."""
        with pytest.raises(ValueError, match="Unknown embedding provider"):
            create_embedding_provider(
                provider="invalid",
                model_name="test"
            )


class TestOpenAIProvider:
    """Test OpenAI embedding provider."""
    
    @pytest.mark.asyncio
    async def test_embed_text(self):
        """Test embedding single text."""
        with patch.dict("os.environ", {"OPENAI_API_KEY": "test-key"}):
            provider = OpenAIEmbeddingProvider("text-embedding-3-small")
            
            # Mock the HTTP client
            mock_response = Mock()
            mock_response.json.return_value = {
                "data": [{"embedding": [0.1, 0.2, 0.3], "index": 0}]
            }
            mock_response.raise_for_status = Mock()
            
            provider.client.post = AsyncMock(return_value=mock_response)
            
            result = await provider.embed_text("test text")
            
            assert result == [0.1, 0.2, 0.3]
            provider.client.post.assert_called_once()
    
    def test_invalid_model(self):
        """Test invalid model raises error."""
        with patch.dict("os.environ", {"OPENAI_API_KEY": "test-key"}):
            with pytest.raises(ValueError, match="Unknown OpenAI model"):
                OpenAIEmbeddingProvider("invalid-model")
    
    def test_missing_api_key(self):
        """Test missing API key raises error."""
        with patch.dict("os.environ", {}, clear=True):
            with pytest.raises(ValueError, match="OpenAI API key not provided"):
                OpenAIEmbeddingProvider("text-embedding-3-small")


"""OpenAI embeddings provider implementation."""

import os

import httpx

from .base import EmbeddingProvider


class OpenAIEmbeddingProvider(EmbeddingProvider):
    """OpenAI embeddings provider using their API."""
    
    # Model dimensions mapping
    MODEL_DIMENSIONS = {
        "text-embedding-3-small": 1536,
        "text-embedding-3-large": 3072,
        "text-embedding-ada-002": 1536,
    }
    
    def __init__(self, model_name: str = "text-embedding-3-small", api_key: str | None = None):
        """Initialize OpenAI embedding provider.
        
        Args:
            model_name: Name of the OpenAI embedding model
            api_key: OpenAI API key (defaults to OPENAI_API_KEY env var)
        """
        if model_name not in self.MODEL_DIMENSIONS:
            raise ValueError(f"Unknown OpenAI model: {model_name}. Supported models: {list(self.MODEL_DIMENSIONS.keys())}")
        
        super().__init__(model_name, self.MODEL_DIMENSIONS[model_name])
        
        self.api_key = api_key or os.environ.get("OPENAI_API_KEY")
        if not self.api_key:
            raise ValueError("OpenAI API key not provided. Set OPENAI_API_KEY environment variable or pass api_key parameter.")
        
        self.client = httpx.AsyncClient(
            base_url="https://api.openai.com/v1",
            headers={
                "Authorization": f"Bearer {self.api_key}",
                "Content-Type": "application/json"
            }
        )
    
    async def embed_text(self, text: str) -> list[float]:
        """Embed a single text using OpenAI API.
        
        Args:
            text: Text to embed
            
        Returns:
            Embedding vector
        """
        embeddings = await self.embed_batch([text])
        return embeddings[0]
    
    async def embed_batch(self, texts: list[str]) -> list[list[float]]:
        """Embed multiple texts using OpenAI API.
        
        Args:
            texts: List of texts to embed
            
        Returns:
            List of embedding vectors
        """
        if not texts:
            return []
        
        response = await self.client.post(
            "/embeddings",
            json={
                "input": texts,
                "model": self.model_name,
                "encoding_format": "float"
            }
        )
        response.raise_for_status()
        
        data = response.json()
        
        # Sort by index to ensure correct order
        embeddings = sorted(data["data"], key=lambda x: x["index"])
        return [item["embedding"] for item in embeddings]
    
    @property
    def provider_name(self) -> str:
        """Get the provider name."""
        return "openai"
    
    async def close(self):
        """Close the HTTP client."""
        await self.client.aclose()